diff -ruN uppsrc/ide/ide.lay uppsrc/ide/ide.lay
--- uppsrc/ide/ide.lay	2011-10-10 00:19:07 +0600
+++ uppsrc/ide/ide.lay	2011-10-19 14:16:32 +0600
@@ -622,6 +622,7 @@
 	ITEM(EditString, folder, LeftPosZ(80, 312).TopPosZ(8, 19))
 	ITEM(Label, dv___2, SetLabel(t_("Import files")).LeftPosZ(8, 72).TopPosZ(32, 19))
 	ITEM(EditString, files, LeftPosZ(80, 312).TopPosZ(32, 19))
+	ITEM(Option, preserve, SetLabel(t_("Preserve directories")).LeftPosZ(8, 192).TopPosZ(56, 19))
 	ITEM(Button, ok, SetLabel(t_("OK")).LeftPosZ(260, 64).TopPosZ(60, 24))
 	ITEM(Button, cancel, SetLabel(t_("Cancel")).LeftPosZ(328, 64).TopPosZ(60, 24))
 END_LAYOUT
diff -ruN uppsrc/ide/UppDlg.h uppsrc/ide/UppDlg.h
--- uppsrc/ide/UppDlg.h	2011-10-07 17:52:46 +0600
+++ uppsrc/ide/UppDlg.h	2011-10-19 14:01:33 +0600
@@ -217,8 +217,9 @@
 	void AddItem(const String& name, bool separator, bool readonly);
 	void AddTopicGroup();
 	void AddSeparator();
-	void Import();
 	void DoImport(const String& dir, const String& mask, bool sep, Progress& pi);
+	void DoImportTree(const String& dir, const String& mask, bool sep, int from, Progress& pi);
+	void Import();
 	void RemoveFile();
 	void DelFile();
 	void RenameFile();
diff -ruN uppsrc/ide/UppWspc.cpp uppsrc/ide/UppWspc.cpp
--- uppsrc/ide/UppWspc.cpp	2011-10-14 19:15:25 +0600
+++ uppsrc/ide/UppWspc.cpp	2011-10-19 14:07:25 +0600
@@ -454,13 +454,15 @@
 		f.separator = f.readonly = true;
 		sep = false;
 	}
+#ifndef PLATFORM_WIN32
 	Sort(files);
 	Sort(dirs);
+#endif
 	for(int i = 0; i < files.GetCount(); i++) {
 		if(pi.StepCanceled())
 			throw String();
 		String name = GetFileName(files[i]);
-		SaveFile(SourcePath(active, name), LoadFile(files[i]));
+		FileCopy(files[i], SourcePath(active, name));
 		Package::File& f = actual.file.Add();
 		f = name;
 		f.separator = f.readonly = false;
@@ -469,6 +471,47 @@
 		DoImport(dirs[i], mask, true, pi);
 }
 
+void WorkspaceWork::DoImportTree(const String& dir, const String& mask, bool sep, int from, Progress& pi)
+{
+	String active = GetActivePackage();
+	if(active.IsEmpty()) return;
+	FindFile ff(AppendFileName(dir, "*.*"));
+	Vector<String> dirs, files;
+	while(ff) {
+		String p = AppendFileName(dir, ff.GetName());
+		if(ff.IsFile() && PatternMatchMulti(mask, ff.GetName()))
+			files.Add(p);
+		if(ff.IsFolder())
+			dirs.Add(p);
+		ff.Next();
+	}
+	const String relPath(dir.Mid(from)),
+		absPath = SourcePath(active, relPath);
+	if(sep && files.GetCount()) {
+		if (!DirectoryExists(absPath))
+			RealizeDirectory(absPath);
+		Package::File& f = actual.file.Add();
+		f = relPath;
+		f.separator = f.readonly = true;
+		sep = false;
+	}
+#ifndef PLATFORM_WIN32
+	Sort(files);
+	Sort(dirs);
+#endif
+	for(int i = 0; i < files.GetCount(); i++) {
+		if(pi.StepCanceled())
+			throw String();
+		const String name = GetFileName(files[i]);
+		FileCopy(files[i], AppendFileName(absPath, name));
+		Package::File& f = actual.file.Add();
+		f = AppendFileName(relPath, name);
+		f.separator = f.readonly = false;
+	}
+	for(int i = 0; i < dirs.GetCount(); i++)
+		DoImportTree(dirs[i], mask, true, from, pi);
+}
+
 void WorkspaceWork::Import()
 {
 	String active = GetActivePackage();
@@ -479,10 +522,20 @@
 	Progress pi("Importing file %d");
 	int fci = filelist.GetCursor();
 	int cs = filelist.GetSbPos();
-	try {
-		DoImport(~dlg.folder, ~dlg.files, false, pi);
+	const String dir = ~dlg.folder;
+	if (~dlg.preserve) {
+		const int from = dir.EndsWith(AsString(DIR_SEP)) ? dir.GetCount() : dir.GetCount() + 1;
+		try {
+			DoImportTree(dir, ~dlg.files, false, from, pi);
+		}
+		catch(String) {}
+	}
+	else {
+		try {
+			DoImport(dir, ~dlg.files, false, pi);
+		}
+		catch(String) {}
 	}
-	catch(String) {}
 	SaveLoadPackage();
 	filelist.SetSbPos(cs);
 	filelist.SetCursor(fci >= 0 ? fci : filelist.GetCount() - 1);
